@@ -4,6 +4,7 @@ use crate::explorer;
 use crate::input::Key;
 use crate::lua;
 use crate::permissions::Permissions;
+use crate::runner;
 use crate::ui::Layout;
 use anyhow::{bail, Result};
 use chrono::{DateTime, Local};
@@ -960,27 +961,25 @@ impl ExplorerConfig {
 pub enum ExternalMsg {
     /// Explore the present working directory and register the filtered nodes.
     /// This operation is expensive. So, try to avoid using it too often.
-    /// Once exploration is done, it will auto `Refresh` the state.
     ExplorePwd,
 
     /// Explore the present working directory and register the filtered nodes asynchronously.
     /// This operation happens asynchronously. That means, the xplr directory buffers won't be updated
     /// immediately. Hence, it needs to be used with care and probably with special checks in place.
     /// To explore `$PWD` synchronously, use `ExplorePwd` instead.
-    /// Once exploration is done, it will auto `Refresh` the state.
     ExplorePwdAsync,
 
     /// Explore the present working directory along with its parents and register the filtered nodes.
     /// This operation happens asynchronously. That means, the xplr directory buffers won't be updated
     /// immediately. Hence, it needs to be used with care and probably with special checks in place.
     /// To explore just the `$PWD` synchronously, use `ExplorePwd` instead.
-    /// Once exploration is done, it will auto `Refresh` the state.
     ExploreParentsAsync,
 
-    /// Refresh the app state (uncluding UI).
+    /// Refresh the UI.
     /// But it will not re-explore the directory if the working directory is the same.
     /// If there is some change in the working directory and you want to re-explore it,
     /// use the `Explore` message instead.
+    /// Also, it will not clear the screen. Use `ClearScreen` for that.
     Refresh,
 
     /// Clears the screen.
@@ -1104,7 +1103,6 @@ pub enum ExternalMsg {
     PopMode,
 
     /// Switch layout.
-    /// This will call `Refresh` automatically.
     ///
     /// > **NOTE:** To be specific about which layout to switch to, use `SwitchLayoutBuiltin` or
     /// `SwitchLayoutCustom` instead.
@@ -1113,13 +1111,11 @@ pub enum ExternalMsg {
     SwitchLayout(String),
 
     /// Switch to a builtin layout.
-    /// This will call `Refresh` automatically.
     ///
     /// **Example:** `SwitchLayoutBuiltin: default`
     SwitchLayoutBuiltin(String),
 
     /// Switch to a custom layout.
-    /// This will call `Refresh` automatically.
     ///
     /// **Example:** `SwitchLayoutCustom: my_custom_layout`
     SwitchLayoutCustom(String),
@@ -1128,7 +1124,7 @@ pub enum ExternalMsg {
     /// Note that the arguments will be shell-escaped.
     /// So to read the variables, the `-c` option of the shell
     /// can be used.
-    /// You may need to pass `Refresh` or `Explore` depening on the expectation.
+    /// You may need to pass `ExplorePwd` depening on the expectation.
     ///
     /// **Example:** `Call: {command: bash, args: ["-c", "read -p test"]}`
     Call(Command),
@@ -1139,15 +1135,18 @@ pub enum ExternalMsg {
     /// **Example:** `CallSilently: {command: tput, args: ["bell"]}`
     CallSilently(Command),
 
-    /// Call a Lua function
+    /// Call a Lua function.
+    /// A `CallLuaArg` object will be passed to the function as argument.
+    /// The function can optionally return a list of messages for xplr to handle
+    /// after the executing the function.
     ///
-    /// **Example:** `CallLua: custom.foo_funtion`
+    /// **Example:** `CallLua: custom.some_custom_funtion`
     CallLua(String),
 
     /// Like `CallLua` but without the flicker. The stdin, stdout
     /// stderr will be piped to null. So it's non-interactive.
     ///
-    /// **Example:** `CallLuaSilently: custom.bar_function`
+    /// **Example:** `CallLuaSilently: custom.some_custom_function`
     CallLuaSilently(String),
 
     /// An alias to `Call: {command: bash, args: ["-c", "${command}"], silent: false}`
@@ -1264,6 +1263,15 @@ pub enum ExternalMsg {
     /// Clear all the node sorters.
     ClearNodeSorters,
 
+    /// Enable mouse
+    EnableMouse,
+
+    /// Disable mouse
+    DisableMouse,
+
+    /// Toggle mouse
+    ToggleMouse,
+
     /// Log information message.
     ///
     /// **Example:** `LogInfo: launching satellite`
@@ -1352,6 +1360,9 @@ pub enum MsgOut {
     CallLua(String),
     CallLuaSilently(String),
     Enque(Task),
+    EnableMouse,
+    DisableMouse,
+    ToggleMouse,
 }
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
@@ -1456,6 +1467,24 @@ impl History {
     }
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize)]
+pub struct CallLuaArg {
+    pub version: String,
+    pub config: Config,
+    pub pwd: String,
+    pub focused_node: Option<Node>,
+    pub directory_buffer: Option<DirectoryBuffer>,
+    pub selection: IndexSet<Node>,
+    pub mode: Mode,
+    pub layout: Layout,
+    pub input_buffer: Option<String>,
+    pub pid: u32,
+    pub session_path: String,
+    pub explorer_config: ExplorerConfig,
+    pub history: History,
+    pub last_modes: Vec<Mode>,
+}
+
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct App {
     pub version: String,
@@ -1503,9 +1532,7 @@ impl App {
                 .to_owned()
                 .unwrap_or_else(|| "default".into()),
         ) {
-            Some(m) => m
-                .clone()
-                .sanitized(config.general().read_only().unwrap_or_default()),
+            Some(m) => m.clone().sanitized(config.general().read_only()),
             None => {
                 bail!("'default' mode is missing")
             }
@@ -1534,7 +1561,7 @@ impl App {
             .to_string();
 
         let mut explorer_config = ExplorerConfig::default();
-        if !config.general().show_hidden().unwrap_or_default() {
+        if !config.general().show_hidden() {
             explorer_config.filters.replace(NodeFilterApplicable::new(
                 NodeFilter::RelativePathDoesNotStartWith,
                 ".".into(),
@@ -1591,10 +1618,11 @@ impl App {
     }
 
     pub fn handle_task(self, task: Task) -> Result<Self> {
-        match task.msg {
-            MsgIn::Internal(msg) => self.handle_internal(msg),
-            MsgIn::External(msg) => self.handle_external(msg, task.key),
-        }
+        let app = match task.msg {
+            MsgIn::Internal(msg) => self.handle_internal(msg)?,
+            MsgIn::External(msg) => self.handle_external(msg, task.key)?,
+        };
+        app.refresh()
     }
 
     fn handle_internal(self, msg: InternalMsg) -> Result<Self> {
@@ -1605,7 +1633,7 @@ impl App {
     }
 
     fn handle_external(self, msg: ExternalMsg, key: Option<Key>) -> Result<Self> {
-        if self.config().general().read_only().unwrap_or_default() && !msg.is_read_only() {
+        if self.config().general().read_only() && !msg.is_read_only() {
             self.log_error("Cannot call shell command in read-only mode.".into())
         } else {
             match msg {
@@ -1687,6 +1715,9 @@ impl App {
                 ExternalMsg::ReverseNodeSorters => self.reverse_node_sorters(),
                 ExternalMsg::ResetNodeSorters => self.reset_node_sorters(),
                 ExternalMsg::ClearNodeSorters => self.clear_node_sorters(),
+                ExternalMsg::EnableMouse => self.enable_mouse(),
+                ExternalMsg::DisableMouse => self.disable_mouse(),
+                ExternalMsg::ToggleMouse => self.toggle_mouse(),
                 ExternalMsg::LogInfo(l) => self.log_info(l),
                 ExternalMsg::LogSuccess(l) => self.log_success(l),
                 ExternalMsg::LogWarning(l) => self.log_warning(l),
@@ -1730,7 +1761,6 @@ impl App {
                     ExternalMsg::LogWarning(
                         "Key map not found. Let's calm down, escape, and try again.".into(),
                     ),
-                    ExternalMsg::Refresh,
                 ]
             });
 
@@ -1744,8 +1774,9 @@ impl App {
     pub fn explore_pwd(self) -> Result<Self> {
         let dir = explorer::explore_sync(
             self.explorer_config().clone(),
-            self.pwd().clone(),
-            self.focused_node().map(|n| n.absolute_path().clone()),
+            self.pwd().into(),
+            self.focused_node().map(|n| n.absolute_path().into()),
+            self.directory_buffer().map(|d| d.focus()).unwrap_or(0),
         )?;
         self.add_directory(dir.parent().clone(), dir)
     }
@@ -1786,10 +1817,8 @@ impl App {
                     self.history = history.push(n.absolute_path().clone())
                 }
             }
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_last(mut self) -> Result<Self> {
@@ -1804,19 +1833,15 @@ impl App {
             if let Some(n) = dir.focused_node() {
                 self.history = history.push(n.absolute_path().clone());
             }
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_previous(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = dir.focus.max(1) - 1;
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_previous_by_relative_index(mut self, index: usize) -> Result<Self> {
@@ -1830,10 +1855,8 @@ impl App {
             if let Some(n) = self.focused_node() {
                 self.history = history.push(n.absolute_path().clone());
             }
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_previous_by_relative_index_from_input(self) -> Result<Self> {
@@ -1847,10 +1870,8 @@ impl App {
     fn focus_next(mut self) -> Result<Self> {
         if let Some(dir) = self.directory_buffer_mut() {
             dir.focus = (dir.focus + 1).min(dir.total.max(1) - 1);
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_next_by_relative_index(mut self, index: usize) -> Result<Self> {
@@ -1864,10 +1885,8 @@ impl App {
             if let Some(n) = self.focused_node() {
                 self.history = history.push(n.absolute_path().clone());
             }
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_next_by_relative_index_from_input(self) -> Result<Self> {
@@ -1944,7 +1963,7 @@ impl App {
                     .map(|s| self.clone().change_directory(s, false))
                     .unwrap_or(Ok(self))
             } else {
-                self.clone().focus_path(target, false)?.refresh()
+                self.clone().focus_path(target, false)
             }
         } else {
             Ok(self)
@@ -1958,7 +1977,7 @@ impl App {
             self.input_buffer = Some(input.to_owned());
         };
         self.logs_hidden = true;
-        self.refresh()
+        Ok(self)
     }
 
     fn buffer_input_from_key(self, key: Option<Key>) -> Result<Self> {
@@ -1972,7 +1991,7 @@ impl App {
     fn set_input_buffer(mut self, string: String) -> Result<Self> {
         self.input_buffer = Some(string);
         self.logs_hidden = true;
-        self.refresh()
+        Ok(self)
     }
 
     fn remove_input_buffer_last_character(mut self) -> Result<Self> {
@@ -1980,10 +1999,8 @@ impl App {
             buf.pop();
             self.input_buffer = Some(buf);
             self.logs_hidden = true;
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn remove_input_buffer_last_word(mut self) -> Result<Self> {
@@ -2001,15 +2018,13 @@ impl App {
 
             self.input_buffer = Some(buf);
             self.logs_hidden = true;
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn reset_input_buffer(mut self) -> Result<Self> {
         self.input_buffer = None;
-        self.refresh()
+        Ok(self)
     }
 
     fn focus_by_index(mut self, index: usize) -> Result<Self> {
@@ -2019,10 +2034,8 @@ impl App {
             if let Some(n) = self.focused_node() {
                 self.history = history.push(n.absolute_path().clone());
             }
-            self.refresh()
-        } else {
-            Ok(self)
-        }
+        };
+        Ok(self)
     }
 
     fn focus_by_index_from_input(self) -> Result<Self> {
@@ -2055,7 +2068,7 @@ impl App {
                         self.history = history.push(n.absolute_path().clone());
                     }
                 }
-                self.refresh()
+                Ok(self)
             } else {
                 self.log_error(format!("{} not found in $PWD", name))
             }
@@ -2113,7 +2126,7 @@ impl App {
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
-                .sanitized(self.config().general().read_only().unwrap_or_default());
+                .sanitized(self.config().general().read_only());
             Ok(self)
         } else {
             self.log_error(format!("Mode not found: {}", mode))
@@ -2126,7 +2139,7 @@ impl App {
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
-                .sanitized(self.config().general().read_only().unwrap_or_default());
+                .sanitized(self.config().general().read_only());
             Ok(self)
         } else {
             self.log_error(format!("Builtin mode not found: {}", mode))
@@ -2139,7 +2152,7 @@ impl App {
             self.input_buffer = None;
             self.mode = mode
                 .to_owned()
-                .sanitized(self.config().general().read_only().unwrap_or_default());
+                .sanitized(self.config().general().read_only());
             Ok(self)
         } else {
             self.log_error(format!("Custom mode not found: {}", mode))
@@ -2149,7 +2162,7 @@ impl App {
     fn switch_layout(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get(layout) {
             self.layout = l.to_owned();
-            self.refresh()
+            Ok(self)
         } else {
             self.log_error(format!("Layout not found: {}", layout))
         }
@@ -2158,7 +2171,7 @@ impl App {
     fn switch_layout_builtin(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get_builtin(layout) {
             self.layout = l.to_owned();
-            self.refresh()
+            Ok(self)
         } else {
             self.log_error(format!("Builtin layout not found: {}", layout))
         }
@@ -2167,7 +2180,7 @@ impl App {
     fn switch_layout_custom(mut self, layout: &str) -> Result<Self> {
         if let Some(l) = self.config().layouts().get_custom(layout) {
             self.layout = l.to_owned();
-            self.refresh()
+            Ok(self)
         } else {
             self.log_error(format!("Custom layout not found: {}", layout))
         }
@@ -2213,13 +2226,13 @@ impl App {
 
     pub fn add_directory(mut self, parent: String, dir: DirectoryBuffer) -> Result<Self> {
         self.directory_buffers.insert(parent, dir);
-        self.refresh()
+        Ok(self)
     }
 
     fn select(mut self) -> Result<Self> {
         if let Some(n) = self.focused_node().map(|n| n.to_owned()) {
             self.selection.insert(n);
-            self.refresh()
+            Ok(self)
         } else {
             Ok(self)
         }
@@ -2231,7 +2244,7 @@ impl App {
         let filename = path.file_name().map(|p| p.to_string_lossy().to_string());
         if let (Some(p), Some(n)) = (parent, filename) {
             self.selection.insert(Node::new(p, n));
-            self.refresh()
+            Ok(self)
         } else {
             Ok(self)
         }
@@ -2242,7 +2255,6 @@ impl App {
             d.nodes.clone().into_iter().for_each(|n| {
                 self.selection.insert(n);
             });
-            self.msg_out.push_back(MsgOut::Refresh);
         };
 
         Ok(self)
@@ -2251,14 +2263,12 @@ impl App {
     fn un_select(mut self) -> Result<Self> {
         if let Some(n) = self.focused_node().map(|n| n.to_owned()) {
             self.selection.retain(|s| s != &n);
-            self.msg_out.push_back(MsgOut::Refresh);
         }
         Ok(self)
     }
 
     fn un_select_path(mut self, path: String) -> Result<Self> {
         self.selection.retain(|n| n.absolute_path != path);
-        self.msg_out.push_back(MsgOut::Refresh);
         Ok(self)
     }
 
@@ -2267,7 +2277,6 @@ impl App {
             d.nodes.clone().into_iter().for_each(|n| {
                 self.selection.retain(|s| s != &n);
             });
-            self.msg_out.push_back(MsgOut::Refresh);
         };
 
         Ok(self)
@@ -2306,7 +2315,6 @@ impl App {
 
     fn clear_selection(mut self) -> Result<Self> {
         self.selection.clear();
-        self.msg_out.push_back(MsgOut::Refresh);
         Ok(self)
     }
 
@@ -2353,7 +2361,7 @@ impl App {
     fn reset_node_filters(mut self) -> Result<Self> {
         self.explorer_config.filters.clear();
 
-        if !self.config().general().show_hidden().unwrap_or_default() {
+        if !self.config().general().show_hidden() {
             self.add_node_filter(NodeFilterApplicable::new(
                 NodeFilter::RelativePathDoesNotStartWith,
                 ".".into(),
@@ -2425,6 +2433,21 @@ impl App {
         Ok(self)
     }
 
+    fn enable_mouse(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::EnableMouse);
+        Ok(self)
+    }
+
+    fn disable_mouse(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::DisableMouse);
+        Ok(self)
+    }
+
+    fn toggle_mouse(mut self) -> Result<Self> {
+        self.msg_out.push_back(MsgOut::ToggleMouse);
+        Ok(self)
+    }
+
     pub fn log_info(mut self, message: String) -> Result<Self> {
         self.logs_hidden = false;
         self.logs.push(Log::new(LogLevel::Info, message));
@@ -2754,4 +2777,34 @@ impl App {
     pub fn logs_hidden(&self) -> bool {
         self.logs_hidden
     }
+
+    pub fn to_lua_arg(&self) -> CallLuaArg {
+        CallLuaArg {
+            version: self.version.clone(),
+            config: self.config.clone(),
+            pwd: self.pwd.clone(),
+            focused_node: self.focused_node().cloned(),
+            directory_buffer: self.directory_buffer().cloned(),
+            selection: self.selection.clone(),
+            mode: self.mode.clone(),
+            layout: self.layout.clone(),
+            input_buffer: self.input_buffer.clone(),
+            pid: self.pid,
+            session_path: self.session_path.clone(),
+            explorer_config: self.explorer_config.clone(),
+            history: self.history.clone(),
+            last_modes: self.last_modes.clone(),
+        }
+    }
+
+    pub fn run(self, focused_path: Option<PathBuf>, lua: &mlua::Lua) -> Result<Option<String>> {
+        runner::run(self, focused_path, lua)
+    }
+}
+
+/// Run xplr TUI
+pub fn run(pwd: PathBuf, focused_path: Option<PathBuf>) -> Result<Option<String>> {
+    let lua = mlua::Lua::new();
+    let app = App::create(pwd, &lua)?;
+    app.run(focused_path, &lua)
 }
